/* 
Read in the area-level dataset:
County = unique area identifier.
y = measure of wind erosion in Iowa counties.
Erodibility = the only auxiliary variable.
Samvar = sampling variances, assumed known.
*/
libname SAE101 "H:\ClassesTaught\SAE101\Jan2013";


* Set an initial value for model error variance;
data work.InitModelVar;
	input est;
	datalines;
	.02
run;
* Set the known sampling error variances,
  and save their number so we later tell PROC MIXED which variances to hold constant;
* (If any observations are missing their samvars, set them to an arbitrary value
   instead of '.' for missing data.
   If you put in a samvar value when y is missing,
   PROC MIXED will still give the right estimate,
   but if you put in '.' for missing data, it will complain.);
data work.Samvars;
	set SAE101.erosion end=eof;
	est = Samvar;
	keep est;
	if eof then call symput('lastsamvar',_N_+1);
run;
* Combine error variances into one dataset;
data work.InitVars;
	set work.InitModelVar work.Samvars;
run;


* Compute the estimates;
* Also look at residual plots, esp. Marginal Studentized Residuals;
* The 'noprofile' and 'hold' allow us to keep the samvars constant
  so that only the model variance is re-estimated and updated;
* The 'int' in the 'random' statement says all the county random effects
  are intercepts with no coefficients and a single common variance;
ods listing exclude dimensions modelinfo nobs classlevels parmsearch lrt
IterHistory ConvergenceStatus covparms covb fitstatistics tests3;
ods output covparms=Covparmestimates;
ods graphics on;
proc mixed data=SAE101.erosion method=reml noprofile;
class county;
model y = Erodibility / s outp=Mixoutp ddfm=residual residual;
random int / subject=county;
repeated / group=county;
parms / parmsdata=InitVars hold=2 to &lastsamvar.;
run;
ods graphics off;
ods listing;
ods output close;
* Look at the estimated model error variance;
proc print data=covparmestimates (obs=1);format estimate 12.10;run;


* Merge and print the results;
data results(drop=samvar);
  retain county erodibility direct_est sae_est direct_stderr sae_stderr direct_cv sae_cv;
  merge mixoutp(rename=(pred=sae_est stderrpred=sae_stderr)  keep=county pred stderrpred)
		sae101.erosion(rename=(y=direct_est));
  by county;
  direct_stderr=sqrt(samvar);
  direct_cv = direct_stderr/direct_est;
  SAE_CV = sae_StdErr/sae_est;
  label sae_est="SAE Estimate"
		sae_stderr="SAE Std Err";
run;
proc print data=results noobs;
  var county direct_est sae_est direct_stderr sae_stderr direct_cv sae_cv;
  format direct_est 6.2 sae_est 6.2 
		direct_stderr 6.2 sae_stderr 6.2 direct_cv 6.2 sae_cv 6.2;
run;

